import { getKysely } from "./kysely.ts";
import { jsonArrayFrom } from "kysely/helpers/sqlite";
import type { DocumentAccessor } from "../model/doc.ts";
import type {
	Document,
	QueryListOption,
	db,
	DocumentBody
} from "dbtype";
import type { NotNull } from "kysely";
import { MyParseJSONResultsPlugin } from "./plugin.ts";
import { getContentFileConstructor } from "src/content/file.ts";
import { join } from "path";

type DBDocument = db.Document;

export type DBTagContentRelation = {
	doc_id: number;
	tag_name: string;
};

class SqliteDocumentAccessor implements DocumentAccessor {
	constructor(private kysely = getKysely()) {
	}
	async search(search_word: string): Promise<Document[]> {
		throw new Error("Method not implemented.");
	}
	async addList(content_list: DocumentBody[]): Promise<number[]> {
		if (content_list.length === 0) return [];
		if (content_list.length > 256) {
			// if content_list is too long, split it.
			const chunkLength = 256;
			const chunked = [];
			for (let i = 0; i < content_list.length; i += chunkLength) {
				chunked.push(content_list.slice(i, i + chunkLength));
			}
			const ids = [];
			for (const chunk of chunked) {
				ids.push(await this.addList(chunk));
			}
			return ids.flat();
		}
		return await this.kysely.transaction().execute(async (trx) => {
			// add tags
			const tagCollected = new Set<string>();
			for (const content of content_list) {
				for (const tag of content.tags) {
					tagCollected.add(tag);
				}
			}
			await trx.insertInto("tags")
				.values(Array.from(tagCollected).map((x) => ({ name: x })))
				.onConflict((oc) => oc.doNothing())
				.execute();

			const ids = await trx.insertInto("document")
				.values(content_list.map((content) => {
					const { tags, additional, ...rest } = content;
					return {
						additional: JSON.stringify(additional),
						created_at: Date.now(),
						...rest,
					};
				}))
				.returning("id")
				.execute();
			const id_lst = ids.map((x) => x.id);

			const doc_tags = content_list.flatMap((content, index) => {
				const { tags, ...rest } = content;
				return tags.map((tag) => ({ doc_id: id_lst[index], tag_name: tag }));
			});
			await trx.insertInto("doc_tag_relation")
				.values(doc_tags)
				.execute();
			return id_lst;
		});
	}
	async add(c: DocumentBody) {
		return await this.kysely.transaction().execute(async (trx) => {
			const { tags, additional, ...rest } = c;
			const id_lst = await trx.insertInto("document").values({
				additional: JSON.stringify(additional),
				created_at: Date.now(),
				...rest,
			})
				.returning("id")
				.executeTakeFirst() as { id: number };
			const id = id_lst.id;

			if (tags.length > 0) {
				// add tags
				await trx.insertInto("tags")
					.values(tags.map((x) => ({ name: x })))
					// on conflict is supported in sqlite and postgresql.
					.onConflict((oc) => oc.doNothing())
					.execute();
			}

			if (tags.length > 0) {
				await trx.insertInto("doc_tag_relation")
					.values(tags.map((x) => ({ doc_id: id, tag_name: x })))
					.execute();
			}
			return id;
		});
	}
	async del(id: number) {
		// delete tags
		await this.kysely
			.deleteFrom("doc_tag_relation")
			.where("doc_id", "=", id)
			.execute();
		// delete document
		const result = await this.kysely
			.deleteFrom("document")
			.where("id", "=", id)
			.executeTakeFirst();
		return result.numDeletedRows > 0;
	}
	async findById(id: number, tagload?: boolean): Promise<Document | undefined> {
		const doc = await this.kysely.selectFrom("document")
			.selectAll()
			.where("id", "=", id)
			.$if(tagload ?? false, (qb) =>
				qb.select(eb => jsonArrayFrom(
					eb.selectFrom("doc_tag_relation")
						.select(["doc_tag_relation.tag_name"])
						.whereRef("document.id", "=", "doc_tag_relation.doc_id")
						.select("tag_name")
				).as("tags")).withPlugin(new MyParseJSONResultsPlugin("tags"))
			)
			.executeTakeFirst();
		if (!doc) return undefined;

		return {
			...doc,
			content_hash: doc.content_hash ?? "",
			additional: doc.additional !== null ? JSON.parse(doc.additional) : {},
			tags: doc.tags?.map((x: { tag_name: string }) => x.tag_name) ?? [],
		};
	}
	async findDeleted(content_type: string) {
		const docs = await this.kysely
			.selectFrom("document")
			.selectAll()
			.where("content_type", "=", content_type)
			.where("deleted_at", "is not", null)
			.$narrowType<{ deleted_at: NotNull }>()
			.execute();
		return docs.map((x) => ({
			...x,
			tags: [],
			content_hash: x.content_hash ?? "",
			additional: {},
		}));
	}
	async findList(option?: QueryListOption) {
		const {
			allow_tag = [],
			eager_loading = true,
			limit = 20,
			use_offset = false,
			offset = 0,
			word,
			content_type,
			cursor,
		} = option ?? {};

		const result = await this.kysely
			.selectFrom("document")
			.selectAll()
			.$if(allow_tag.length > 0, (qb) => {
				return allow_tag.reduce((prevQb, tag, index) => {
					return prevQb.innerJoin(`doc_tag_relation as tags_${index}`, `tags_${index}.doc_id`, "document.id")
						.where(`tags_${index}.tag_name`, "=", tag);
				}, qb) as unknown as typeof qb;
			})
			.$if(word !== undefined, (qb) => qb.where("title", "like", `%${word}%`))
			.$if(content_type !== undefined, (qb) => qb.where("content_type", "=", content_type as string))
			.$if(use_offset, (qb) => qb.offset(offset))
			.$if(!use_offset && cursor !== undefined, (qb) => qb.where("id", "<", cursor as number))
			.limit(limit)
			.$if(eager_loading, (qb) => {
				return qb.select(eb =>
					eb.selectFrom(e =>
						e.selectFrom("doc_tag_relation")
							.select(["doc_tag_relation.tag_name"])
							.whereRef("document.id", "=", "doc_tag_relation.doc_id")
							.as("agg")
					).select(e => e.fn.agg<string>("json_group_array", ["agg.tag_name"])
						.as("tags_list")
					).as("tags")
				)
			})
			.orderBy("id", "desc")
			.execute();
		return result.map((x) => ({
			...x,
			content_hash: x.content_hash ?? "",
			additional: x.additional !== null ? (JSON.parse(x.additional)) : {},
			tags: JSON.parse(x.tags ?? "[]"), //?.map((x: { tag_name: string }) => x.tag_name) ?? [],
		}));
	}
	async findByGidList(gid_list: number[]) {
		const result = await this.kysely
			.selectFrom("document")
			.selectAll()
			.where("gid", "in", gid_list)
			.execute();
		return result.map((x) => ({
			...x,
			content_hash: x.content_hash ?? "",
			tags: [],
			additional: {},
		}));
	}
	async findByPath(path: string, filename?: string): Promise<Document[]> {
		const results = await this.kysely
			.selectFrom("document")
			.selectAll()
			.where("basepath", "=", path)
			.$if(filename !== undefined, (qb) => qb.where("filename", "=", filename as string))
			.execute();
		return results.map((x) => ({
			...x,
			content_hash: x.content_hash ?? "",
			tags: [],
			additional: {},
		}));
	}

	async update(c: Partial<Document> & { id: number }) {
		const { id, tags, additional, ...rest } = c;
		const r = await this.kysely.updateTable("document")
			.set({
				...rest,
				modified_at: Date.now(),
				additional: additional !== undefined ? JSON.stringify(additional) : undefined,
			})
			.where("id", "=", id)
			.executeTakeFirst();
		return r.numUpdatedRows > 0;
	}
	async addTag(c: Document, tag_name: string) {
		if (c.tags.includes(tag_name)) return false;
		await this.kysely.insertInto("tags")
			.values({ name: tag_name })
			.onConflict((oc) => oc.doNothing())
			.execute();
		await this.kysely.insertInto("doc_tag_relation")
			.values({ tag_name: tag_name, doc_id: c.id })
			.execute();
		c.tags.push(tag_name);
		return true;
	}
	async delTag(c: Document, tag_name: string) {
		if (c.tags.includes(tag_name)) return false;
		await this.kysely.deleteFrom("doc_tag_relation")
			.where("tag_name", "=", tag_name)
			.where("doc_id", "=", c.id)
			.execute();
		c.tags.splice(c.tags.indexOf(tag_name), 1);
		return true;
	}

	async getSimilarDocument(doc: Document): Promise<Document[]> {
		const tags = doc.tags.filter((x) => !x.startsWith("type:")
			&& x !== "series:original");
		if (tags.length === 0) return [];
		// get similar document based on tags count.
		const results = await this.kysely
			.selectFrom("document as doc")
			.innerJoin("doc_tag_relation", "doc_tag_relation.doc_id", "doc.id")
			.where("doc_tag_relation.tag_name", "in", tags)
			.where("doc.id", "<>", doc.id)
			.selectAll()
			.select(qb => ([
				qb.fn.count("doc_tag_relation.tag_name").as("tag_count"),

				qb.selectFrom(e =>
					e.selectFrom("doc_tag_relation")
						.select(["doc_tag_relation.tag_name"])
						.whereRef("doc.id", "=", "doc_tag_relation.doc_id")
						.as("agg")
				).select(e => e.fn.agg<string>("json_group_array", ["agg.tag_name"])
					.as("tags_list")
				).as("tags")
			]))
			.groupBy("doc.id")
			.orderBy("tag_count", "desc")
			.limit(10)
			.execute();
		// TODO: tf-idf or other similarity calculation
		return results.map((x) => ({
			...x,
			tags: JSON.parse(x.tags ?? "[]"),
			additional: {},
		}));
	}

	async rescanDocument(c: Document) {
		return this.kysely.transaction().execute(async (trx) => {
			const ContentFile = getContentFileConstructor(c.content_type);
			if (ContentFile === undefined) {
				throw new Error(`${c.content_type} is not defined.`);
			}
			const path = join(c.basepath, c.filename);
			const content = new ContentFile(path);
			const body = await content.createDocumentBody();

			const { tags, ...rest } = body;

			// update document
			await trx.updateTable("document")
				.set({
					...rest,
					modified_at: Date.now(),
					additional: JSON.stringify(body.additional),
				})
				.where("id", "=", c.id)
				.execute();

			// delete old tags
			await trx.deleteFrom("doc_tag_relation")
				.where("doc_id", "=", c.id)
				.execute();
			// update tags and doc_tag_relation
			if (tags.length > 0) {
				await trx.insertInto("tags")
					.values(tags.map((x) => ({ name: x })))
					.onConflict((oc) => oc.doNothing())
					.execute();
			}
			await trx.insertInto("doc_tag_relation")
				.values(tags.map((x) => ({ tag_name: x, doc_id: c.id }))
				)
				.execute();

		});
	}
}
export const createSqliteDocumentAccessor = (kysely = getKysely()): DocumentAccessor => {
	return new SqliteDocumentAccessor(kysely);
};
